/*-----------------------------------------
   Calc.C -- Calculation module
   Version date: 99.10.23
  -----------------------------------------*/

#include <windows.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include "External.h"

#define PI                 3.141592748
#define DEGTORAD           PI/180.0
#define SAILSTEP           0.5
#define JIB                12
#define RUDDER_INDEX       13
#define TOTAL_THRUST_INDEX 14
#define TOTAL_DRAG_INDEX   15
#define NET_THRUST_INDEX   16
#define HEEL_INDEX         17

// Prototypes:
void          Calculate() ;

extern void   CanShadow () ;
extern void   PlanView () ;
extern void   SailPlan () ;
extern void   ShowMessage() ;
extern int    Sgn (float) ;

/* --------------------------------------
   Calculate: performs all cyclical calculations
   Version date = 99.09.23
----------------------------------------*/

void Calculate()
  {
  float        fAlongShip ;
  float        fAngle ;
  float        fArea ;
  float        fCrossShip ;
  float        fDeltaHdg ;
  float        fDeltaSpeed ;
  float        fDist ;
  float        fDistance ;
  float        fEffPerpArea ;     
  float        fFactor ;
  float        fH ;
  int          iI ;
  int          iJ ;
  static int   iLastCOEHeight ;
  static int   iLastCOELeeward ;
  static int   iLastHeelingAngle ;
  float        fLeftEdge ;
  float        fLeftFactor [13] ;
  float        fMoment ;
  int          iN ;
  float        fNewLeftEdge ;
  float        fOfset ;
  float        fParalax ;
  float        fRightEdge ;
  float        fRightFactor [13] ;
  float        fSail ;
  BOOL         bShad ;
  float        fShad ;
  float        fShadLeftEdge ;
  float        fShadRightEdge ;
  int          cSh ;
  int          iSign ;
  int          cSl ;
  float        fSlantLeft, fSlantRight ;
  float        fSlHeight ;
  float        fSpeedEast ;
  float        fSpeedNorth ;
  BOOL         bSplit ;
  float        fTemp ;
  float        fTemp2 ;
  float        fTotalPerpArea ;
  float        fTotalThrust ;

  // clock
  fNow = (float) clock()/CLOCKS_PER_SEC ;
  fIncrement = (fNow - fLastTime) ;
  fLastTime = fNow ;

  // Calculate relative wind angle
  // negative = port, positive = starboard, 0 = astern, 180 = ahead
  fRelWindAngle = fHeading - (fWindDirection + 180.0) ;
  if (fRelWindAngle < -180.0)
    fRelWindAngle = fRelWindAngle + 360.0 ;
  if (fRelWindAngle > 180.0)
    fRelWindAngle = fRelWindAngle - 360.0 ;

  // Calculate apparent wind vector
  fAlongShip = fWindSpeed * cos(fRelWindAngle * DEGTORAD) - fSpeed ;
  fCrossShip = fWindSpeed * sin(fRelWindAngle * DEGTORAD) + fLeewaySpeed ;
  if (fabs(fAlongShip) <= 0.1)
    // avoid division by zero
    fAppWindAngle = 90 ;
  else
    fAppWindAngle = atan(fabs(fCrossShip/fAlongShip)) * 180.0 / PI ;
  if (fAlongShip < 0 )
    fAppWindAngle = 180 - fAppWindAngle ;
  fAppWindAngle = fAppWindAngle * Sgn(fRelWindAngle) ;
  if (fabs(fAppWindAngle - fOldAppWindAngle) > 1)
    {
    //update PlanView
    fOldAppWindAngle = fAppWindAngle ;
    bPlanViewIsChanged = TRUE ;
    }
  fAppWindSpeed = sqrt(fCrossShip * fCrossShip + fAlongShip * fAlongShip) ;
  fAppWindDirection = fHeading - (180.0 + fAppWindAngle) ;
  if (fAppWindDirection < 0)
    fAppWindDirection = fAppWindDirection + 360.0 ;

  // check if jib still pulling
  if (Sgn(fSailAngle[JIB]) == -Sgn(fAppWindAngle))
    // jib flips over
    fSailAngle[JIB] = -fSailAngle[JIB] ;

  //Calculate the apparent distances of the masts and the corners of the sails
  //  from the centre of rotation when viewed from the direction of the wind
  fDist = sin(fAppWindAngle * DEGTORAD) ;
  for (cSl = 0 ; cSl <= 12 ; cSl++)
    {
    if (iSailStatus[cSl] < 4)
      {
      switch (cSl)
        {
        case 0:
        case 1:
        case 2:
        case 3:
          fParalax = fDist * fMastDistance[0] ;
          break ;
        case 4:
        case 5:
        case 6:
        case 7:
          fParalax = fDist * fMastDistance[1] ;
          break ;
        case 8:
        case 9:
        case 10:
        case 11:
          fParalax = fDist * fMastDistance[2] ;
          break ;
        case 12:
          fParalax = fDist * fMastDistance[3] ;
        }

      // calculate Sailplan positions of corners of effective sail
      if (fEffSailHead[cSl] == fSailHead[cSl])
        {
        fEffPlanHeadLeft[cSl] = fPlanHeadLeft[cSl] ;
        fEffPlanHeadRight[cSl] = fPlanHeadRight[cSl] ;
        }
      else
        {
        fTemp = (fEffSailHead[cSl] - fSailFoot[cSl]) * 2.0 ;
        fSlantLeft = (fPlanFootLeft[cSl] - fPlanHeadLeft[cSl]) / 
          ((fSailHead[cSl] - fSailFoot[cSl]) * 2.0) ;
        fSlantRight = (fPlanFootRight[cSl] - fPlanHeadRight[cSl]) / 
          ((fSailHead[cSl] - fSailFoot[cSl]) * 2.0) ;
        fEffPlanHeadLeft[cSl] = fPlanFootLeft[cSl] - fSlantLeft * fTemp ;
        fEffPlanHeadRight[cSl] = fPlanFootRight[cSl] - fSlantRight * fTemp ;
        }

      fFactor = (sin((fAppWindAngle + 90.0 + fSailAngle[cSl]) * DEGTORAD)) ;
      if (cSl == 11) 
        // mizzen course
        fFactor = (sin((fAppWindAngle + fSailAngle[cSl]) * DEGTORAD)) ;
      fWindHeadLeft[cSl] = fParalax + fFactor * fEffPlanHeadLeft[cSl] ;
      fWindHeadRight[cSl] = fParalax + fFactor * fEffPlanHeadRight[cSl] ;
      fWindFootLeft[cSl] = fParalax + fFactor * fPlanFootLeft[cSl] ;
      fWindFootRight[cSl] = fParalax + fFactor * fPlanFootRight[cSl] ;
      if (cSl == JIB) 
        // jib
        {
        fFactor = (sin((fAppWindAngle + fSailAngle[cSl]) * DEGTORAD)) ;
        fWindFootLeft[cSl] = fParalax + fFactor * fPlanFootLeft[cSl] ;
        fFactor = sin(fAppWindAngle * DEGTORAD) ;
        fWindHeadLeft[cSl] = fParalax + fFactor * fEffPlanHeadLeft[cSl] ;
        fWindHeadRight[cSl] = fParalax + fFactor * fEffPlanHeadRight[cSl] ;
        fWindFootRight[cSl] = fParalax + fFactor * fPlanFootRight[cSl] ;
        }
      if (fWindHeadLeft[cSl] > fWindHeadRight[cSl])
        {
        fTemp2 = fWindHeadLeft[cSl] ;
        fWindHeadLeft[cSl] = fWindHeadRight[cSl] ;
        fWindHeadRight[cSl] = fTemp2 ;
        }
      if (fWindFootLeft[cSl] > fWindFootRight[cSl])
        {
        fTemp2 = fWindFootLeft[cSl] ;
        fWindFootLeft[cSl] = fWindFootRight[cSl] ;
        fWindFootRight[cSl] = fTemp2 ;
        }
      }
    }
  //Calculate sail area perpendicular to the wind and draw perspective view
  for (cSl = 0; cSl <= 12; cSl++)
    {
    if (iSailStatus[cSl] < 4)
      {
      fSlHeight = fEffSailHead[cSl] - fSailFoot[cSl] ;
      fLeftFactor[cSl] = (fWindHeadLeft[cSl] - fWindFootLeft[cSl]) / fSlHeight ;
      fRightFactor[cSl] = (fWindHeadRight[cSl] - fWindFootRight[cSl]) / fSlHeight ;
      }
    else
      fExposedFract[cSl] = 0 ;
    }
  if (fPerpFract[7] >= 0.0)  // mainsail drawing
    {
    if (fPerpFract[3] >= 0)
      iSeq = 1 ;
    else
      iSeq = 0 ;
    }
  else    // mainsail aback
    { 
    if (fPerpFract[3] >= 0)
      iSeq = 3 ;
    else
      iSeq = 2 ;
    }
  for (iI = 0; iI<=12; iI++)
    {
    for (iN = 0; iN <= 7; iN ++)
      iDoShad[iI][iN] = -1 ;
    }

  for (iI = 0; iI<=12; iI++)
    {
    cSl = iSequence[iSeq][iI] ;
    fPerpSailArea[cSl] = 0 ;
    if (iSailStatus[cSl] < 4)
      {
      // determine which sails (probably) shadow this sail
      iN = 0 ;
      for (iJ = 0; iJ <= 7; iJ++)
        {
        cSh = iCanShad[iSeq][cSl][iJ] ;
        // cSh=shadowing sail
        if (cSh == -1)
          break ;
        bShad = TRUE ;
        fShad = fWindHeadLeft[cSh] ;
        if (fWindFootLeft[cSh] < fShad)
          fShad = fWindFootLeft[cSh] ;
        fSail = fWindHeadRight[cSl] ;
        if (fWindFootRight[cSl] > fSail)
          fSail = fWindFootRight[cSl] ;
        if (fShad > fSail)
          bShad = FALSE ;
        fShad = fWindHeadRight[cSh] ;
        if (fWindFootRight[cSh] > fShad)
          fShad = fWindFootRight[cSh] ;
        fSail = fWindHeadLeft[cSl] ;
        if (fWindFootLeft[cSl] < fSail)
          fSail = fWindFootLeft[cSl] ;
        if (fShad < fSail)
          bShad = FALSE ;
        if (bShad)
          {
          iDoShad[cSl][iN] = cSh ;
          // in which vertical range does this sail shadow?
          fTemp = 0.0 ;
          if (fSailFoot[cSh] > fSailFoot[cSl])
            fTemp = fSailFoot[cSh] - fSailFoot[cSl] ;
          iShadStart[cSl][iN] = fTemp ;
          fTemp = fEffSailHead[cSl] - fSailFoot[cSl] ;
          if (fEffSailHead[cSh] < fEffSailHead[cSl])
            fTemp = fEffSailHead[cSh] - fSailFoot[cSl] ;
          iShadEnd[cSl][iN] = fTemp ;
          iShadOfset[cSl][iN] = fSailFoot[cSl] - fSailFoot[cSh] ;
          iN = iN + 1 ;
          }
        }
      fTotalPerpArea = 0.0 ;
      fEffPerpArea = 0.0 ;
      fSlHeight = fEffSailHead[cSl] - fSailFoot[cSl] ;

      // now finally calculate the unshadowed perpendicular area in bands of SAILSTEP feet
      for (fH = SAILSTEP / 2 ; fH <= fSlHeight - (SAILSTEP / 2); fH= fH + SAILSTEP)
        {
        // first the total perpendicular width of the band
        fLeftEdge = fWindFootLeft[cSl] + fH * fLeftFactor[cSl] ;
        fRightEdge = fWindFootRight[cSl] + fH * fRightFactor[cSl] ;
        fTotalPerpArea = fTotalPerpArea + (fRightEdge - fLeftEdge) * SAILSTEP ;
        // and now the width of shadowing
  Again:
        bSplit = FALSE ;
        iN = 0 ;
        while (iDoShad[cSl][iN] > -1)
          { 
          if ((fH >= iShadStart[cSl][iN]) && (fH <= iShadEnd[cSl][iN]))
            {
            cSh = iDoShad[cSl][iN] ;
            fOfset = iShadOfset[cSl][iN] ;
            fShadLeftEdge = fWindFootLeft[cSh] + (fH + fOfset) * fLeftFactor[cSh] ;
            fShadRightEdge = fWindFootRight[cSh] + (fH + fOfset) * fRightFactor[cSh] ;
            if (fShadLeftEdge > fShadRightEdge)
              {
              fTemp = fShadLeftEdge ;
              fShadRightEdge = fShadLeftEdge ;
              fShadLeftEdge = fTemp ;
              } 
            if ((fShadLeftEdge > fLeftEdge) && (fShadRightEdge < fRightEdge))
              // no shadow on both edges
              {
              fRightEdge = fShadLeftEdge ;
              bSplit = TRUE ;
              fNewLeftEdge = fShadRightEdge ;
              }
            if ((fShadLeftEdge <= fLeftEdge) && (fShadRightEdge > fLeftEdge))
              // shadow overlaps left edge
              fLeftEdge = fShadRightEdge ;
            if ((fShadLeftEdge < fRightEdge) && (fShadRightEdge >= fRightEdge))            
              //shadow overlaps right edge
              fRightEdge = fShadLeftEdge ;
            if (fLeftEdge > fRightEdge)
              // shadow overlaps both edges, band is totally shadowed
              {
              fRightEdge = fLeftEdge ;
              // no point in continuing
              if (! bSplit)
                break ;
              } 
            }
          iN = iN + 1 ;
          }
        fEffPerpArea = fEffPerpArea + (fRightEdge - fLeftEdge) * SAILSTEP ;
        if (bSplit == TRUE)
          {
          fLeftEdge = fNewLeftEdge ;
          fRightEdge = fWindFootRight[cSl] + fH * fRightFactor[cSl] ;
          goto Again ;
          }
        }

      fPerpSailArea[cSl] = fEffPerpArea ;
      if (fTotalPerpArea == 0 )
        fExposedFract[cSl] = 0.0 ;
      else
        fExposedFract[cSl] = fEffPerpArea / fTotalPerpArea ;
      // correct for heeling
      fPerpSailArea[cSl] = fPerpSailArea[cSl] * cos(fHeelingAngle * DEGTORAD) ;
      }
    }

  // Calculate force acting on each sail and foreward, lateral and tortional thrust
  fForwardThrust = 0.0 ;
  fLeewardThrust = 0.0 ;
  fRotationalThrust = 0.0 ;

  fMoment = 0.0 ;
  fArea = 0.0  ; // to calculate COEForeAft

  for (iI = 0; iI<=12; iI++)
    {
    iSailWarning[iI] = 0 ;
    if (iSailStatus[iI] > 3)
      {
      // furled or blown out
      fPerpFract[iI] = 0.0 ;
      iSailWarning[iI] = 4 ;
      }
    else
      {   
      if ((iI==11) || (iI==12))
        // fore=and-aft sails
        fAngle = -fSailAngle[iI] - fAppWindAngle ;
       else 
         fAngle = (90.0 - fSailAngle[iI]) - fAppWindAngle ;
      // useful fraction
      fPerpFract[iI] = sin(fAngle * DEGTORAD) ;
      // check if sail is ashiver and/or aback
      iSign = Sgn(fPerpFract[iI]) ;
      if ((iI==11) || (iI==12))
        {
        if (fabs(fPerpFract[iI]) < 0.173) 
          {
          // sail ashiver 
          iSailWarning[iI] = 1 ; 
          // reduced efficiency at < 10  (arbitrary cubic function)
          fTemp = fPerpFract[iI] ;
          fPerpFract[iI] = iSign * fTemp * fTemp * fTemp * fTemp / 0.00518 ;
          }
        }
      else 
        { 
        if (fPerpFract[iI] < 0.5)
          {
          if (fPerpFract[iI] > 0.0)
            { 
            // sail ashiver
            iSailWarning[iI] = 1 ;
            // reduced efficiency at < 30  (arbitrary cubic function)
            fTemp = fPerpFract[iI] ;
            fPerpFract[iI] = iSign * fTemp * fTemp * fTemp * fTemp / 0.125 ;
            }
          else
            {  
            if (fPerpFract[iI] > -0.173)
              {
              // sail ashiver and aback
              iSailWarning[iI] = 2 ;
              // reduced efficiency at < 10  (arbitrary cubic function)
              fTemp = fPerpFract[iI] ;
              fPerpFract[iI] = iSign * fTemp * fTemp * fTemp * fTemp / 0.00518 ;
              }
            else
              {
              // sail aback
              iSailWarning[iI] = 3;
              }
            }  
          }
        }
      } 
    // useful force
    fPerpForce[iI] = fPerpSailArea[iI] * fAppWindSpeed * fAppWindSpeed * fPerpFract[iI] / 1000.0 ;

    fAngle = fSailAngle[iI] * DEGTORAD ;
    if ((iI == 11) || (iI == 12))
      // fore-and-aft sails (for which 0= fore-and-aft)
      fAngle = PI / 2 - fAngle ;
    // useful force for headway; + = ahead
    fHeadwayForce[iI] = fPerpForce[iI] * cos(fAngle) ;
    if ((iI == 11) || (iI == 12))
      fHeadwayForce[iI] = -fHeadwayForce[iI] ;
    // Useful force for leeway; +=to starboard 
    fLeewayForce[iI] = fPerpForce[iI] * sin(fAngle) ;
    fForwardThrust = fForwardThrust + fHeadwayForce[iI] ;
    fLeewardThrust = fLeewardThrust + fLeewayForce[iI] ;
    switch (iI)
      {
      case 0:
      case 1:
      case 2:
      case 3:
        fDistance = fMastDistance[0] ;
        break ;
      case 4:
      case 5:
      case 6:
      case 7:
        fDistance = fMastDistance[1] ;
        break ;
      case 8:
      case 9:
      case 10:
        fDistance = fMastDistance[2] ;
        break ;
      case 11:
        fDistance = fMastDistance[2] - (fFootWidth[11] / 2.0) ;
        break ;
      case 12:
        fDistance = fMastDistance[3] ;
      }
    // rotational force
    fRotatForce[iI] = fLeewayForce[iI] * fDistance ;
    fMoment = fMoment + fPerpSailArea[iI] * fDistance ;
    fArea = fArea + fPerpSailArea[iI] ;
    fRotationalThrust = fRotationalThrust + fRotatForce[iI] ;
    }

  if (fArea != 0)
    fCOEForeAft = fMoment / fArea ;

  //Do these calculations for the rudder
  fPerpSailArea[RUDDER_INDEX] = fabs(fRudderArea * sin(iHelmAngle * DEGTORAD)); // effective area
  fPerpFract[RUDDER_INDEX] = fabs(sin(iHelmAngle * DEGTORAD)); // useful fraction
  fPerpForce[RUDDER_INDEX] = fPerpSailArea[RUDDER_INDEX] * fSpeed * fSpeed * 
    fPerpFract[RUDDER_INDEX] * fRudderFactor / 1000.0 ; // useful force
  fHeadwayForce[RUDDER_INDEX] = -Sgn(fSpeed) * fabs(fPerpForce[RUDDER_INDEX] *
    sin(iHelmAngle * DEGTORAD)) ; // useful force for headway
  fLeewayForce[RUDDER_INDEX] = Sgn(fSpeed) * Sgn(iHelmAngle) * fPerpForce[RUDDER_INDEX] *
    cos(iHelmAngle * DEGTORAD) ; // 'useful force for leeway
  fRotatForce[RUDDER_INDEX] = -fLeewayForce[RUDDER_INDEX] * fShipLength / 2.0 ;
  fForwardThrust = fForwardThrust + fHeadwayForce[RUDDER_INDEX] ;
  fLeewardThrust = fLeewardThrust + fLeewayForce[RUDDER_INDEX] ;
  fRotationalThrust = fRotationalThrust + fRotatForce[RUDDER_INDEX] ;

  // Calculate heeling thrust
  fHeelingThrust = 0 ;
  fCOEHeight = 0 ;
  for (iI = 0; iI <= 12 ; iI++)
    {
    fHeelForce[iI] = fMeanSailHeight[iI] * cos(fHeelingAngle * DEGTORAD) * fLeewayForce[iI] / 1000.0 ;
    fHeelingThrust = fHeelingThrust + fHeelForce[iI] ;
    fCOEHeight = fCOEHeight + fLeewayForce[iI] ;
    }
  if (fCOEHeight != 0)
    fCOEHeight = 1000.0 * fHeelingThrust / fCOEHeight ;
  if (fabs(fCOEHeight - iLastCOEHeight) > 5)
    {
    iLastCOEHeight = (int) fCOEHeight ;
    bSternViewIsChanged = TRUE ;
    }

  // Calculate angle of heel
  fRightingMoment = -Sgn(fHeelingAngle) * fabs(sin(fHeelingAngle * 2.0 * DEGTORAD) * 
    fShipMass * fHeelAngleFactor * fMaxHeelMoment) ;
  fResistance = -fHeelResistFactor * Sgn(fDeltaHeel) * fHeelSpeed * fHeelSpeed ;
  fTotalThrust = fHeelingThrust + fRightingMoment + fResistance ;
  fDeltaSpeed = fTotalThrust * fIncrement / fVerticalInertia ;
  // damp excessive heeling acceleration (more than about 2/sec/sec)
  if (fabs(fDeltaSpeed) > 1.0)
    {
    fDeltaSpeed = 1.0 * Sgn(fDeltaSpeed) ;
    bAlertHeel = TRUE ;
    }
  else
    bAlertHeel = FALSE ;
  fHeelSpeed = fHeelSpeed + fDeltaSpeed ;
  fDeltaHeel = fHeelSpeed * fIncrement ;
  fHeelingAngle = fHeelingAngle + fDeltaHeel ;
  if ((int) fHeelingAngle != iLastHeelingAngle)
    {
    iLastHeelingAngle = (int) fHeelingAngle ;
    bSternViewIsChanged = TRUE ;
    }

  // Calculate the contribution of heel to rotational forces

  fCOELeeward = -fCOEHeight * tan(fHeelingAngle * DEGTORAD) ;
  // + = COE is to port
  if ((int) fCOELeeward != iLastCOELeeward)
    {
    iLastCOELeeward = (int) fCOELeeward ;
    bPlanViewIsChanged = TRUE ;
    }
  fEffectOfHeel = fCOELeeward * fForwardThrust ;
  fRotationalThrust = fRotationalThrust + fEffectOfHeel ;

  // Calculate the forward, leeward and rotational drags
  if (fSpeed>0)
    fForwardDrag = (fSpeed * fSpeed) * fCSABow * fWaterDragFactor / 1000.0 ;
  else
    fForwardDrag = (fSpeed * fSpeed) * fCSAStern * fWaterDragFactor / 1000.0 ;
  fForwardDrag = -fForwardDrag * Sgn(fSpeed) ;
  fLeewardDrag = (fLeewaySpeed * fLeewaySpeed) * 
    (fCSASide * fWaterDragFactor * (1 + fabs(fSpeed)) / 2000.0) ;
  // arbitrary additional increase of drag with speed to reduce leeway
  fLeewardDrag = (fLeewaySpeed * fLeewaySpeed) * fCSASide * fWaterDragFactor / 2000.0 ; //######
  fLeewardDrag = -fLeewardDrag * Sgn(fLeewaySpeed) ;
  fRotationalDrag = fShipLength * fShipLength * fShipLength * fShipLength * fShipDraught * 
    fRotationalSpeed * fRotationalSpeed * fRotatDragFactor / (32.0 * 1000.0) ;
  fRotationalDrag = -fRotationalDrag * Sgn(fRotationalSpeed) ;

  // Calculate the forward, leeward and rotational accelerations and speeds
  // ...forward
  fDeltaSpeed = (fForwardThrust + fForwardDrag) * fIncrement / fShipMass ;
  // damp excessive acceleration (more than about 2 kt/sec)
  if (fabs(fDeltaSpeed) > 1.0)
    {
    fDeltaSpeed = 1.0 * Sgn(fDeltaSpeed) ;
    bAlertSpeed = TRUE ;
    } 
  else
    bAlertSpeed = FALSE ;

  fSpeed = fSpeed + fDeltaSpeed ;

  //...leeward
  fDeltaSpeed = (fLeewardThrust + fLeewardDrag) * fIncrement / fShipMass ;
  // damp excessive acceleration to leeway (more than about 2 kt/sec)
  if (fabs(fDeltaSpeed) > 1.0)
    {
    fDeltaSpeed = 1.0 * Sgn(fDeltaSpeed) ;
    bAlertLeeway = TRUE ;
    } 
  else
    bAlertLeeway = FALSE ;

  fLeewaySpeed = fLeewaySpeed + fDeltaSpeed ;
  if (fabs(fSpeed) > 0.1)
    fLeewayAngle = Sgn(fSpeed) * atan(fLeewaySpeed / fSpeed) * 180.0 / PI ;
  else
    fLeewayAngle = Sgn(fLeewaySpeed) * 90.0 ;

  //...rotational
  if (fabs(fRotationalDrag) > fabs(10 * fRotationalThrust))
    fRotationalDrag = Sgn(fRotationalDrag) * 10.0 * fRotationalThrust ;
  fDeltaSpeed = (fRotationalThrust + fRotationalDrag) * fIncrement / fRotationalInertia ;
  fRotationalSpeed = fRotationalSpeed + fDeltaSpeed ;
  if (fRotationalThrust == 0)
    {
    fRotationalDrag = 0 ;
    fRotationalSpeed = 0 ;
    }
  fDeltaHdg = fRotationalSpeed * fIncrement * 180.0 / PI ;
  // damp excessive rotational speed (more than about 2/sec)
  if (fabs(fDeltaHdg) > 1.0)
    {
    fDeltaHdg = 1.0 * Sgn(fDeltaHdg) ;
    bAlertRotat = TRUE ;
    } 
  else
    bAlertRotat = FALSE ;

  //write forces to Sails-array to simplify display procedure
  fHeadwayForce[TOTAL_THRUST_INDEX] = fForwardThrust ;
  fHeadwayForce[TOTAL_DRAG_INDEX] = fForwardDrag ;
  fHeadwayForce[NET_THRUST_INDEX] = fForwardThrust + fForwardDrag ;
  fLeewayForce[TOTAL_THRUST_INDEX] = fLeewardThrust ;
  fLeewayForce[TOTAL_DRAG_INDEX] = fLeewardDrag ;
  fLeewayForce[NET_THRUST_INDEX] = fLeewardThrust + fLeewardDrag ;
  fRotatForce[HEEL_INDEX] = fEffectOfHeel ;
  fRotatForce[TOTAL_THRUST_INDEX] = fRotationalThrust ;
  fRotatForce[TOTAL_DRAG_INDEX] = fRotationalDrag ;
  fRotatForce[NET_THRUST_INDEX] = fRotationalThrust + fRotationalDrag ;
  fHeelForce[TOTAL_THRUST_INDEX] = fHeelingThrust ;
  fHeelForce[TOTAL_DRAG_INDEX] = fRightingMoment ;
  fHeelForce[NET_THRUST_INDEX] = fHeelingThrust + fRightingMoment + fResistance ;

  // Report dampers in action
  //CurrentX = 4: CurrentY = 39: Line (CurrentX, CurrentY)-(CurrentX + 3.8, CurrentY + 1), BackColor, BF
  //ForeColor = QBColor(8)
  //CurrentX = 4: CurrentY = 39: Print Alert_l$
  //ForeColor = QBColor(0)

  // Calculate new heading
  fDegreesPerMin = fDeltaHdg * 60.0 / fIncrement ;
  fHeading = fHeading + fDeltaHdg ;
  if (fHeading > 360.0)
    fHeading = fHeading - 360.0 ;
  if (fHeading <= 0.0)
    fHeading = fHeading + 360.0 ;

  // Check for blown out sails
  for (cSl = 0; cSl <= 12 ;cSl++)
    {
    if ((iSailStatus[cSl] < 4) && (fPerpSailArea[cSl] != 0.0))
      {
      if (fAppWindSpeed > (fMaxSpeed[cSl] / fSetFraction[iSailStatus[cSl]]))
        {
        iSailStatus[cSl] = 5 ;
        sprintf(sMessage, "%s blown out", sSailName[cSl]) ; 
        ShowMessage () ; 
        fMaxSpeed[cSl] = fTheorMaxSpeed[cSl] * (0.9 + 0.2 * (float) rand()/RAND_MAX) ;
        if (! bParalax)
          SailPlan () ;
          CanShadow () ;
        if (cSl == 13)
          PlanView() ;
        }
      }
    } 

  // Calculate new position
  fSpeedNorth = fSpeed * cos(fHeading * DEGTORAD) + fLeewaySpeed * cos((fHeading+90.) * DEGTORAD) ;
  fSpeedEast = fSpeed * sin(fHeading * DEGTORAD) + fLeewaySpeed * sin((fHeading+90.) * DEGTORAD) ;
  fLatitude = fLatitude + fSpeedNorth * fIncrement / 3600.0 ;
  fLongtitude = fLongtitude + fSpeedEast * fIncrement / 3600.0 ;

  // Update cycle count
  iCycles = iCycles +1 ;
  }

